@@ -14,24 +14,25 @@ cpy_compile = compile
 
 
 class Source(object):
-    """ a immutable object holding a source code fragment,
+    """ an immutable object holding a source code fragment,
         possibly deindenting it.
     """
+
     _compilecounter = 0
 
     def __init__(self, *parts, **kwargs):
         self.lines = lines = []
-        de = kwargs.get('deindent', True)
-        rstrip = kwargs.get('rstrip', True)
+        de = kwargs.get("deindent", True)
+        rstrip = kwargs.get("rstrip", True)
         for part in parts:
             if not part:
                 partlines = []
-            if isinstance(part, Source):
+            elif isinstance(part, Source):
                 partlines = part.lines
             elif isinstance(part, (tuple, list)):
                 partlines = [x.rstrip("\n") for x in part]
             elif isinstance(part, six.string_types):
-                partlines = part.split('\n')
+                partlines = part.split("\n")
                 if rstrip:
                     while partlines:
                         if partlines[-1].strip():
@@ -60,7 +61,7 @@ class Source(object):
             if key.step not in (None, 1):
                 raise IndexError("cannot slice a Source with a step")
             newsource = Source()
-            newsource.lines = self.lines[key.start:key.stop]
+            newsource.lines = self.lines[key.start : key.stop]
             return newsource
 
     def __len__(self):
@@ -79,7 +80,7 @@ class Source(object):
         source.lines[:] = self.lines[start:end]
         return source
 
-    def putaround(self, before='', after='', indent=' ' * 4):
+    def putaround(self, before="", after="", indent=" " * 4):
         """ return a copy of the source object with
             'before' and 'after' wrapped around it.
         """
@@ -90,7 +91,7 @@ class Source(object):
         newsource.lines = before.lines + lines + after.lines
         return newsource
 
-    def indent(self, indent=' ' * 4):
+    def indent(self, indent=" " * 4):
         """ return a copy of the source object with
             all lines indented by the given indent-string.
         """
@@ -98,14 +99,14 @@ class Source(object):
         newsource.lines = [(indent + line) for line in self.lines]
         return newsource
 
-    def getstatement(self, lineno, assertion=False):
+    def getstatement(self, lineno):
         """ return Source statement which contains the
             given linenumber (counted from 0).
         """
-        start, end = self.getstatementrange(lineno, assertion)
+        start, end = self.getstatementrange(lineno)
         return self[start:end]
 
-    def getstatementrange(self, lineno, assertion=False):
+    def getstatementrange(self, lineno):
         """ return (start, end) tuple which spans the minimal
             statement region which containing the given lineno.
         """
@@ -131,13 +132,7 @@ class Source(object):
         """ return True if source is parseable, heuristically
             deindenting it by default.
         """
-        try:
-            import parser
-        except ImportError:
-            def syntax_checker(x):
-                return compile(x, 'asd', 'exec')
-        else:
-            syntax_checker = parser.suite
+        from parser import suite as syntax_checker
 
         if deindent:
             source = str(self.deindent())
@@ -145,7 +140,7 @@ class Source(object):
             source = str(self)
         try:
             # compile(source+'\n', "x", "exec")
-            syntax_checker(source + '\n')
+            syntax_checker(source + "\n")
         except KeyboardInterrupt:
             raise
         except Exception:
@@ -156,9 +151,14 @@ class Source(object):
     def __str__(self):
         return "\n".join(self.lines)
 
-    def compile(self, filename=None, mode='exec',
-                flag=generators.compiler_flag,
-                dont_inherit=0, _genframe=None):
+    def compile(
+        self,
+        filename=None,
+        mode="exec",
+        flag=generators.compiler_flag,
+        dont_inherit=0,
+        _genframe=None,
+    ):
         """ return compiled code object. if filename is None
             invent an artificial filename which displays
             the source/line position of the caller frame.
@@ -170,20 +170,20 @@ class Source(object):
             base = "<%d-codegen " % self._compilecounter
             self.__class__._compilecounter += 1
             if not filename:
-                filename = base + '%s:%d>' % (fn, lineno)
+                filename = base + "%s:%d>" % (fn, lineno)
             else:
-                filename = base + '%r %s:%d>' % (filename, fn, lineno)
-        source = "\n".join(self.lines) + '\n'
+                filename = base + "%r %s:%d>" % (filename, fn, lineno)
+        source = "\n".join(self.lines) + "\n"
         try:
             co = cpy_compile(source, filename, mode, flag)
         except SyntaxError:
             ex = sys.exc_info()[1]
             # re-represent syntax errors from parsing python strings
-            msglines = self.lines[:ex.lineno]
+            msglines = self.lines[: ex.lineno]
             if ex.offset:
-                msglines.append(" " * ex.offset + '^')
+                msglines.append(" " * ex.offset + "^")
             msglines.append("(code was compiled probably from here: %s)" % filename)
-            newex = SyntaxError('\n'.join(msglines))
+            newex = SyntaxError("\n".join(msglines))
             newex.offset = ex.offset
             newex.lineno = ex.lineno
             newex.text = ex.text
@@ -195,12 +195,15 @@ class Source(object):
             linecache.cache[filename] = (1, None, lines, filename)
             return co
 
+
 #
 # public API shortcut functions
 #
 
 
-def compile_(source, filename=None, mode='exec', flags=generators.compiler_flag, dont_inherit=0):
+def compile_(
+    source, filename=None, mode="exec", flags=generators.compiler_flag, dont_inherit=0
+):
     """ compile the given source to a raw code object,
         and maintain an internal cache which allows later
         retrieval of the source code for the code object
@@ -219,9 +222,10 @@ def getfslineno(obj):
     """ Return source location (path, lineno) for the given object.
     If the source cannot be determined return ("", -1)
     """
-    import _pytest._code
+    from .code import Code
+
     try:
-        code = _pytest._code.Code(obj)
+        code = Code(obj)
     except TypeError:
         try:
             fn = inspect.getsourcefile(obj) or inspect.getfile(obj)
@@ -241,6 +245,7 @@ def getfslineno(obj):
     assert isinstance(lineno, int)
     return fspath, lineno
 
+
 #
 # helper functions
 #
@@ -259,12 +264,13 @@ def findsource(obj):
 
 
 def getsource(obj, **kwargs):
-    import _pytest._code
-    obj = _pytest._code.getrawcode(obj)
+    from .code import getrawcode
+
+    obj = getrawcode(obj)
     try:
         strsrc = inspect.getsource(obj)
     except IndentationError:
-        strsrc = "\"Buggy python version consider upgrading, cannot get source\""
+        strsrc = '"Buggy python version consider upgrading, cannot get source"'
     assert isinstance(strsrc, str)
     return Source(strsrc, **kwargs)
 
@@ -285,14 +291,14 @@ def deindent(lines, offset=None):
 
     def readline_generator(lines):
         for line in lines:
-            yield line + '\n'
-        while True:
-            yield ''
+            yield line + "\n"
 
     it = readline_generator(lines)
 
     try:
-        for _, _, (sline, _), (eline, _), _ in tokenize.generate_tokens(lambda: next(it)):
+        for _, _, (sline, _), (eline, _), _ in tokenize.generate_tokens(
+            lambda: next(it)
+        ):
             if sline > len(lines):
                 break  # End of input reached
             if sline > len(newlines):
@@ -308,19 +314,20 @@ def deindent(lines, offset=None):
     except (IndentationError, tokenize.TokenError):
         pass
     # Add any lines we didn't see. E.g. if an exception was raised.
-    newlines.extend(lines[len(newlines):])
+    newlines.extend(lines[len(newlines) :])
     return newlines
 
 
 def get_statement_startend2(lineno, node):
     import ast
+
     # flatten all statements and except handlers into one lineno-list
     # AST's line numbers start indexing at 1
     values = []
     for x in ast.walk(node):
-        if isinstance(x, ast.stmt) or isinstance(x, ast.ExceptHandler):
+        if isinstance(x, (ast.stmt, ast.ExceptHandler)):
             values.append(x.lineno - 1)
-            for name in "finalbody", "orelse":
+            for name in ("finalbody", "orelse"):
                 val = getattr(x, name, None)
                 if val:
                     # treat the finally/orelse part as its own statement
@@ -338,11 +345,8 @@ def get_statement_startend2(lineno, node):
 def getstatementrange_ast(lineno, source, assertion=False, astnode=None):
     if astnode is None:
         content = str(source)
-        try:
-            astnode = compile(content, "source", "exec", 1024)  # 1024 for AST
-        except ValueError:
-            start, end = getstatementrange_old(lineno, source, assertion)
-            return None, start, end
+        astnode = compile(content, "source", "exec", 1024)  # 1024 for AST
+
     start, end = get_statement_startend2(lineno, astnode)
     # we need to correct the end:
     # - ast-parsing strips comments
@@ -374,38 +378,3 @@ def getstatementrange_ast(lineno, source, assertion=False, astnode=None):
         else:
             break
     return astnode, start, end
-
-
-def getstatementrange_old(lineno, source, assertion=False):
-    """ return (start, end) tuple which spans the minimal
-        statement region which containing the given lineno.
-        raise an IndexError if no such statementrange can be found.
-    """
-    # XXX this logic is only used on python2.4 and below
-    # 1. find the start of the statement
-    from codeop import compile_command
-    for start in range(lineno, -1, -1):
-        if assertion:
-            line = source.lines[start]
-            # the following lines are not fully tested, change with care
-            if 'super' in line and 'self' in line and '__init__' in line:
-                raise IndexError("likely a subclass")
-            if "assert" not in line and "raise" not in line:
-                continue
-        trylines = source.lines[start:lineno + 1]
-        # quick hack to prepare parsing an indented line with
-        # compile_command() (which errors on "return" outside defs)
-        trylines.insert(0, 'def xxx():')
-        trysource = '\n '.join(trylines)
-        #              ^ space here
-        try:
-            compile_command(trysource)
-        except (SyntaxError, OverflowError, ValueError):
-            continue
-
-        # 2. find the end of the statement
-        for end in range(lineno + 1, len(source) + 1):
-            trysource = source[start:end]
-            if trysource.isparseable():
-                return start, end
-    raise SyntaxError("no valid source range around line %d " % (lineno,))
